# test/test_suite.py

from sqlalchemy.testing.suite import *
from sqlalchemy.inspection import inspect
import sqlalchemy.testing as testing
from sqlalchemy.testing.assertions import eq_
from sqlalchemy.testing.suite import ComponentReflectionTest as _ComponentReflectionTest
"""
Copyright 2019 Splice Machine Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

"""
Overriden test cases from
SQLAlchemy

Running Test Cases 
-----------------
cd /path/to/spliceengine/
./start-splice-cluster # fresh db instance (no tables besides defaults)
./sqlshell.sh 
splice> CREATE SCHEMA test_schema;
<ctrl+c>

cd /path/to/package/dir
sudo pip install . pytest pyodbc
pytest -vv .
"""

class ComponentReflectionTest(_ComponentReflectionTest):
    def _test_get_foreign_keys(self, schema=None):
        # we just need to lowercase the assertion columns
        # in this test -- test is intact otherwise
        meta = self.metadata
        users, addresses, dingalings = (
            self.tables.users,
            self.tables.email_addresses,
            self.tables.dingalings,
        )
        insp = inspect(meta.bind)
        expected_schema = schema
        # users
        if testing.requires.self_referential_foreign_keys.enabled:
            users_fkeys = insp.get_foreign_keys(users.name, schema=schema)
            fkey1 = users_fkeys[0]
            if schema:
                eq_(fkey1["referred_schema"].lower(), expected_schema)
            else:
                eq_(fkey1["referred_schema"], None)
            eq_(fkey1["referred_table"].lower(), users.name)
            eq_(fkey1["referred_columns"][0].lower(), "user_id")
            if testing.requires.self_referential_foreign_keys.enabled:
                eq_(fkey1["constrained_columns"][0].lower(), "parent_user_id")

        # addresses
        addr_fkeys = insp.get_foreign_keys(addresses.name, schema=schema)
        fkey1 = addr_fkeys[0]
        if schema:
            eq_(fkey1["referred_schema"].lower(), expected_schema)
        else:
            eq_(fkey1["referred_schema"], None)

        eq_(fkey1["referred_table"].lower(), users.name)
        eq_(len(fkey1["referred_columns"]), 1)
        eq_(len(fkey1["constrained_columns"]), 1)

    def _test_get_pk_constraint(self, schema=None):
        # we just need to lowercase the assertion columns
        # in this test -- test is intact otherwise
        meta = self.metadata
        users, addresses = self.tables.users, self.tables.email_addresses
        insp = inspect(meta.bind)

        users_cons = insp.get_pk_constraint(users.name, schema=schema)
        users_pkeys = users_cons["constrained_columns"]
        eq_(users_pkeys[0].lower(), "user_id")

        addr_cons = insp.get_pk_constraint(addresses.name, schema=schema)
        addr_pkeys = addr_cons["constrained_columns"]
        eq_(addr_pkeys[0].lower(), "address_id")

    def test_get_table_names_fk(self):
        self.test_get_table_names()
    
    def test_get_table_names_fks(self):
        self.test_get_table_names()



import re

"""
Copyright 2019 Splice Machine Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# keywords that trigger double quotation with "<identifier>"
# otherwise splice will interpret them as SQL keywords
RESERVED_WORDS = {'activate', 'disallow', 'locale', 'result', 'add', 'disconnect', 'localtime', 'result_set_locator', 'size',
                  'after', 'distinct', 'localtimestamp', 'return', 'alias', 'do', 'locator', 'returns', 'all', 'double',
                  'locators', 'revoke', 'allocate', 'drop', 'lock', 'right', 'allow', 'dssize', 'lockmax', 'rollback',
                  'alter', 'dynamic', 'locksize', 'routine', 'and', 'each', 'long', 'row', 'any', 'editproc', 'loop',
                  'row_number', 'as', 'else', 'maintained', 'rownumber', 'asensitive', 'elseif', 'materialized', 'rows',
                  'associate', 'enable', 'maxvalue', 'rowset', 'asutime', 'encoding', 'microsecond', 'rrn', 'at',
                  'encryption', 'microseconds', 'run', 'attributes', 'end', 'minute', 'savepoint', 'audit', 'end-exec',
                  'minutes', 'schema', 'authorization', 'ending', 'minvalue', 'scratchpad', 'aux', 'erase', 'mode',
                  'scroll', 'auxiliary', 'escape', 'modifies', 'search', 'before', 'every', 'month', 'second', 'begin',
                  'except', 'months', 'seconds', 'between', 'exception', 'new', 'secqty', 'binary', 'excluding',
                  'new_table', 'security', 'bufferpool', 'exclusive', 'nextval', 'select', 'by', 'execute', 'no',
                  'sensitive', 'cache', 'exists', 'nocache', 'sequence', 'call', 'exit', 'nocycle', 'session', 'called',
                  'explain', 'nodename', 'session_user', 'capture', 'external', 'nodenumber', 'set', 'cardinality',
                  'extract', 'nomaxvalue', 'signal', 'cascaded', 'fenced', 'nominvalue', 'simple', 'case', 'fetch',
                  'none', 'some', 'cast', 'fieldproc', 'noorder', 'source', 'ccsid', 'file', 'normalized', 'specific',
                  'char', 'final', 'not', 'sql', 'character', 'for', 'null', 'sqlid', 'check', 'foreign', 'nulls',
                  'stacked', 'close', 'free', 'numparts', 'standard', 'cluster', 'from', 'obid', 'start', 'collection',
                  'full', 'of', 'starting', 'collid', 'function', 'old', 'statement', 'column', 'general', 'old_table',
                  'static', 'comment', 'generated', 'on', 'stay', 'commit', 'get', 'open', 'stogroup', 'concat',
                  'global', 'optimization', 'stores', 'condition', 'go', 'optimize', 'style', 'connect', 'goto',
                  'option', 'substring', 'connection', 'grant', 'or', 'summary', 'constraint', 'graphic', 'order',
                  'synonym', 'contains', 'group', 'out', 'sysfun', 'continue', 'handler', 'outer', 'sysibm', 'count',
                  'hash', 'over', 'sysproc', 'count_big', 'hashed_value', 'overriding', 'system', 'create', 'having',
                  'package', 'system_user', 'cross', 'hint', 'padded', 'table', 'current', 'hold', 'pagesize',
                  'tablespace', 'current_date', 'hour', 'parameter', 'then', 'current_lc_ctype', 'hours', 'part',
                  'time', 'current_path', 'identity', 'partition', 'timestamp', 'current_schema', 'if', 'partitioned',
                  'to', 'current_server', 'immediate', 'partitioning', 'transaction', 'current_time', 'in',
                  'partitions', 'trigger', 'current_timestamp', 'including', 'password', 'trim', 'current_timezone',
                  'inclusive', 'path', 'type', 'current_user', 'increment', 'piecesize', 'undo', 'cursor', 'index',
                  'plan', 'union', 'cycle', 'indicator', 'position', 'unique', 'data', 'inherit', 'precision', 'until',
                  'database', 'inner', 'prepare', 'update', 'datapartitionname', 'inout', 'prevval', 'usage',
                  'datapartitionnum', 'insensitive', 'primary', 'user', 'second', 'date', 'insert', 'priqty', 'using',
                  'day', 'integrity', 'privileges', 'validproc', 'days', 'intersect', 'procedure', 'value',
                  'db2general', 'into', 'program', 'values', 'db2genrl', 'is', 'psid', 'variable', 'db2sql', 'isobid',
                  'query', 'variant', 'dbinfo', 'isolation', 'queryno', 'vcat', 'dbpartitionname', 'iterate', 'range',
                  'version', 'dbpartitionnum', 'jar', 'rank', 'view', 'deallocate', 'java', 'read', 'volatile',
                  'declare', 'join', 'reads', 'volumes', 'default', 'key', 'recovery', 'when', 'defaults', 'label',
                  'references', 'whenever', 'definition', 'language', 'referencing', 'where', 'delete', 'lateral',
                  'refresh', 'while', 'dense_rank', 'lc_ctype', 'release', 'with', 'denserank', 'leave', 'rename',
                  'without', 'describe', 'left', 'repeat', 'wlm', 'descriptor', 'like', 'reset', 'write',
                  'deterministic', 'linktype', 'resignal', 'xmlelement', 'diagnostics', 'local', 'restart', 'year',
                  'disable', 'localdate', 'restrict', 'years', '', 'abs', 'grouping', 'regr_intercept', 'are', 'int',
                  'regr_r2', 'array', 'integer', 'regr_slope', 'asymmetric', 'intersection', 'regr_sxx', 'atomic',
                  'interval', 'regr_sxy', 'avg', 'large', 'regr_syy', 'bigint', 'first', 'last', 'leading', 'rollup',
                  'blob', 'ln', 'scope', 'boolean', 'lower', 'similar', 'both', 'match', 'smallint', 'ceil', 'max',
                  'specifictype', 'ceiling', 'member', 'sqlexception', 'char_length', 'merge', 'sqlstate',
                  'character_length', 'method', 'sqlwarning', 'clob', 'min', 'sqrt', 'coalesce', 'mod', 'stddev_pop',
                  'collate', 'module', 'stddev_samp', 'collect', 'multiset', 'submultiset', 'convert', 'national',
                  'sum', 'corr', 'natural', 'symmetric', 'corresponding', 'nchar', 'tablesample', 'covar_pop', 'nclob',
                  'timezone_hour', 'covar_samp', 'normalize', 'timezone_minute', 'cube', 'nullif', 'trailing',
                  'cume_dist', 'numeric', 'translate', 'current_default_transform_group', 'octet_length', 'translation',
                  'current_role', 'only', 'treat', 'current_transform_group_for_type', 'overlaps', 'true', 'dec',
                  'overlay', 'uescape', 'decimal', 'percent_rank', 'unknown', 'deref', 'percentile_cont', 'unnest',
                  'element', 'percentile_disc', 'upper', 'exec', 'power', 'var_pop', 'exp', 'real', 'var_samp', 'false',
                  'recursive', 'varchar', 'filter', 'ref', 'varying', 'float', 'regr_avgx', 'width_bucket', 'floor',
                  'regr_avgy', 'window', 'fusion', 'regr_count', 'within', 'asc'}

# case insensitive reserved words regular expression
# for efficient matching (w/o iteration)
# we need [1:] to remove first pipe from string
# or it will match everything
RESERVED_WORDS_REGEX = re.compile('|'.join(RESERVED_WORDS)[1:], re.IGNORECASE)



#!/usr/bin/env python

from setuptools import setup
from setuptools.command.install import install as Install
from platform import system
from subprocess import check_call as run_bash

"""
Copyright 2019 Splice Machine Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

VERSION = '0.1.4.12'
ODBC_VERSION = '2.8.66.0'

def bash(command):
    run_bash(command.split())

class CustomInstall(Install):
    linux_url = 'https://splice-releases.s3.amazonaws.com/odbc-driver/Linux64/splice_odbc_linux64-2.8.66.0.tar.gz'
    window_url = 'https://splice-releases.s3.amazonaws.com/odbc-driver/Win64/splice_odbc_setup_64bit_2.8.66.0.msi'
    mac_url = 'https://splice-releases.s3.amazonaws.com/odbc-driver/MacOSX64/splice_odbc_macosx64-2.8.66.0.tar.gz'
    nix_url = {'Darwin': mac_url, 'Linux': linux_url}
    def nix_hanlder(self):
        url = CustomInstall.nix_url[system()]
        file_name = url.split('/')[-1]
        bash('mkdir -p /tmp')
        bash('curl -kLs {url} -o /tmp/{file_name}'.format(url=url,file_name=file_name))
        bash('tar -xzf /tmp/{} -C /tmp'.format(file_name))
        driver_name = 'libsplice_odbc64.dylib' if system()=='Darwin' else 'lib64/libsplice_odbc.so'
        driver_location = '/Library/ODBC/SpliceMachine/' if system()=='Darwin' else '/usr/local/splice'
        file_name = file_name.rstrip('.tar.gz')
        bash('mkdir -p {}'.format(driver_location))
        bash('mv -f /tmp/{file_name}/{driver_name} {driver_location}'.format(file_name=file_name, driver_name=driver_name,
                                                                          driver_location=driver_location))

    def windows_handler(self):
        pass

    def run(self):
        Install.run(self)
        {'Linux': self.nix_hanlder,
         'Windows': self.windows_handler,
         'Darwin': self.nix_hanlder}[system()]()


setup(
    name='splicemachinesa',
    version=VERSION,
    url='https://splicemachine.com',
    keywords=['sqlalchemy', 'splice', 'machine', 'splice machine', 'odbc', 'driver', 'sqlalchemy driver'],
    license='Apache License 2.0',
    description='SQLAlchemy support for Splice Machine RDBMS',
    author='Amrit Baveja',
    author_email='abaveja@splicemachine.com',
    download_url='https://splice-releases.s3.amazonaws.com/splice-sqlalchemy/splicemachinesa-{version}.dev1.tar.gz'.format(
	version=VERSION),
    platforms='All',
    install_requires=[
        'sqlalchemy',
        'pyodbc>=4.0.26'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Topic :: Database'
    ],
    packages=['splicemachinesa'],
    entry_points={
        'sqlalchemy.dialects': [
            'splicemachinesa=splicemachinesa.pyodbc:SpliceMachineDialect_pyodbc',
            'splicemachinesa.splice_machine=splicemachinesa.splice_machine:SpliceMachineDialect_sm',
            'splicemachinesa.pyodbc=splicemachinesa.pyodbc:SpliceMachineDialect_pyodbc',
        ]
    },
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    zip_safe=False,
    tests_require=['nose >= 0.11'],
    cmdclass={'install': CustomInstall}
)



"""
Copyright 2019 Splice Machine Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

"""
External utilities
to make using the SpliceMachineSa
driver easier
"""


def url_builder(odbc_driver_location, host='0.0.0.0', port=1527, user='splice', password='admin'):
    """
    Build the SpliceMachineSA DB URL for SQLAlchemy create_engine

    :param odbc_driver_location: (str) the local path to the platform
        specific ODBC driver (found in splice-odbc bucket in S3)
    :param host: (str) the Splice Machine database host
    :param port: (int) the port on which Splice Machine is running
    :param user: (str) the username for the database
    :param password: (str) the password for the database

    :returns: (str) create_engine connection string
    """
    return 'splicemachinesa+pyodbc:///?DRIVER={driver_loc}&PORT={port}\
&PWD={password}&UID={user}&URL={host}'.format(
        driver_loc=odbc_driver_location, port=port,
        password=password, user=user, host=host
    )
